<?php

/**
 * @package plugin ScriptsDown
 * @copyright (C) 2010-2012 RicheyWeb - www.richeyweb.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 *
 */
// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

/**
 * ScriptsDown system plugin
 */
class plgSystemScriptsDown extends JPlugin {

	private $_scripts = array();
	private $_script = '';
	private $_mediaVersion;

	function onBeforeCompileHead() {
		if (JFactory::getApplication()->isAdmin())
		{
			return;
		}
		$doc = JFactory::getDocument();
		$this->_removeScripts($doc);
		if($this->params->get('declarations',false)) {
			$this->_script = $doc->_script['text/javascript'];
			unset($doc->_script['text/javascript']);
		}
	}

	function onAfterRender() {
		if(!count($this->_scripts)) {
			return;
		}
		$doc = JFactory::getDocument();
		$this->_mediaVersion = '?'.$doc->getMediaVersion();
		$this->_moveScripts($doc);
		if($this->params->get('declarations',false)) {
			$this->_moveScript($doc);
		}
	}
	
	private function _removeScripts($doc) {
		$regex = $this->_prepExclude((array) $this->params->get('scripts', array()));
		$excluded = array();
		foreach ($regex as $r)
		{
			$exclude = preg_grep($r, array_keys($doc->_scripts));
			$excluded = array_merge($excluded, $exclude);
		}
		foreach ($doc->_scripts as $src => $attribs)
		{
			if (!in_array($src, $excluded))
			{
				$this->_scripts[$src] = $attribs;
				unset($doc->_scripts[$src]);
			}
		}
	}

	private function _moveScripts() {
		$app = JFactory::getApplication();
		$body = $app->getBody();
		foreach ($this->_scripts as $src => $attribs)
		{
			$body = str_replace('</body>',$this->_renderScript($src, $attribs)."</body>",$body);
		}
		$app->setBody($body);
	}
	
	private function _moveScript($doc) {
		$app = JFactory::getApplication();
		$body = str_replace('</body>',$this->_renderDeclaration($this->_script)."</body>",$app->getBody());		
		$app->setBody($body);
	}
	
	private function _renderDeclaration($script) {
		$tag = $this->_renderScript(false,array('type'=>'text/javascript'));
		return str_replace('</script>',"\n".$script."\n</script>",$tag);
	}

	private function _renderScript($src=false, $attribs) {
		$defaultJsMimes = array('text/javascript', 'application/javascript', 'text/x-javascript', 'application/x-javascript');
		$doc = JFactory::getDocument();
		$mediaVersion = (isset($attribs['options']['version']) && $attribs['options']['version'] && strpos($src, '?') === false && ($this->_mediaVersion || $attribs['options']['version'] !== 'auto'))?$this->_mediaVersion:'';
		$dom = new DOMDocument('1.0','UTF-8');
		$script = $dom->createElement('script');
		// src attribute
		if($src) {
			$this->_addAttribute($dom,$script,'src',$src);
		}
		// type attribute
		if (array_intersect(array_keys($attribs), array('type', 'mime')) && !$doc->isHtml5() && in_array(($attribs['type']?:$attribs['mime']), $defaultJsMimes)) {
			$this->_addAttribute($dom,$script,'type',$attribs['type']);
		}
		// defer attribute
		if(array_key_exists($attribs,'defer') && $attribs['defer'] === true) {
			$this->_addAttribute($dom,$script,'defer');
		}
		// async attribute
		if(array_key_exists($attribs,'async') && $attribs['async'] === true) {
			$this->_addAttribute($dom,$script,'asnyc');
		}
		// charset attribute
		if(array_key_exists($attribs,'charset')) {
			$charsetattr = $dom->createAttribute('charset');
			$this->_addAttribute($dom,$script,'charset',$attribs['charset']);
		}
		
		if(isset($attribs['options']) && isset($attribs['options']['conditional'])) {
			$dom->appendChild($dom->createComment('[if ' .$attribs['options']['conditional'].']'));
			$dom->appendChild($script);
			$dom->appendChild($dom->createComment('[endif]'));
		} else {
			$dom->appendChild($script);
		}
		return $dom->saveHTML();
	}
	
	private function _addAttribute($dom,&$element,$name,$value=false) {
		$attr = $dom->createAttribute($name);
		if($value) {
			$attr->value = $value;
		}
		$element->appendChild($attr);
	}

	private function _prepExclude($a) {
		return array_values(array_map(function($i)
			{
				return $i->regex;
			}, $a));
	}

}
